/*
	NUMERICKE METODY I
	
	Hledani komplexnich korenu polynomialni rovnice P_n(z)=0

	Autor: Jan Schee, jan.schee@fpf.slu.cz
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>


#define DCMPLX double complex

void poly(DCMPLX z, double a[], int n, DCMPLX p[])
{
	int i;
	

	p[0]=a[n]+I*0;
	
	for(i=n-1;i>=0;i--)
	{
		p[0]=p[0]*z;
		p[0]=p[0]+a[i];
	}

	p[1]=n*a[n];

	for(i=n-1;i>=1;i--)
	{
		p[1]=p[1]*z;
		p[1]=p[1]+i*a[i];
	}

	p[2]=n*(n-1)*a[n];
	
	for(i=n-1;i>=2;i--)
	{
		p[2]=p[2]*z;
		p[2]=p[2]+i*(i-1)*a[i];
	}

}

DCMPLX laguerre(double a[], int n)
{
	DCMPLX z;
	DCMPLX H,G,G1,G2;
	DCMPLX p[3];
	const double eps=1e-8;
	
	int i;

	z=0+I*0;

	for(i=0;i<100;i++)
	{
	
		poly(z,a,n,p);

		if(cabs(p[0])<eps) break;

		H=(n-1)+I*0;
		H=H*((DCMPLX)(n-1)*p[1]*p[1]-n*p[0]*p[2]);

	
		G1=p[1]+csqrt(H);
		G2=p[1]-csqrt(H);

		if(cabs(G1)>cabs(G2)) G=G1;
		else G=G2;	

		z=z-n*p[0]/G;
	}	

	return z;
}


void poldiv(double a[], int n, DCMPLX z1, double v[])
{
		int i;
		DCMPLX p;

		p=a[n];

		for(i=n-1;i>=0;i--)
		{
			v[i]=creal(p);
			p=p*z1+a[i];
		}
}

void poldiv1(double a[], int n, DCMPLX z1, double v[])
{
	int i;
	DCMPLX p;
	DCMPLX *tmp;
	DCMPLX *tmp1;
	DCMPLX z2;

	tmp =(DCMPLX *)malloc((n+1)*sizeof(DCMPLX));
	tmp1=(DCMPLX *)malloc((n+1)*sizeof(DCMPLX));



	p=a[n];
	for(i=n-1;i>=0;i--)
	{
		tmp[i]=p;
		p=p*z1+a[i];
	}

	

	z2=conj(z1);

	p=tmp[n-1];
	for(i=n-2;i>=0;i--)
	{
		tmp1[i]=p;
		p=p*z2+tmp[i];
	}
	
	for(i=0;i<=n-2;i++) v[i]=creal(tmp1[i]);
		
	free(tmp);
	free(tmp1);
}



int main()
{
	const int n=5; 
	int i,j;	
	double a[n+1];
	double b1[n+1];
	double b2[n+1];
	DCMPLX p[3],z[5+1];

	a[0]=6;
	a[1]=0;
	a[2]=-3;
	a[3]=2;
	a[4]=1;
	a[5]=-9.0;

	for(i=0;i<=n;i++) b1[i]=a[i];

	for(i=n;i>=1;i--)
	{	
		z[i]=laguerre(b1,i);
		if(cimag(z[i])>1e-15){
			poldiv1(b1,i,z[i],b2);
		
			z[i-1]=conj(z[i]);

			i--;
		}else poldiv(b1,i,z[i],b2);

		for(j=0;j<=i-1;j++) b1[j]=b2[j];
	}
	
	for(i=1;i<=n;i++){
		 poly(z[i],a,n,p);
		 printf("z[%d]=%lf + I(%lf) ... (P=%lf)\n",i,creal(z[i]),cimag(z[i]),cabs(p[0]));		

	}

	return 0;
}
