#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.1415927

#define FUNC(f) int (*f)(double , const double * , double *, int, void * )

typedef struct{
	double M;
	double G;
}PARAMS;

typedef struct{
	double *k1,*k2,*k3,*k4;
	double *ytmp;	
}WORKSPACE;


int func(double t, const double *y, double *f, int n, void *p)
{
	int i;
	PARAMS *par=(PARAMS *)p;
	double vx,vy,xx,yy,X;
	double K;

	vx=y[0];
	vy=y[1];
	xx=y[2];
	yy=y[3];

	K=par->G*par->M;
	X=sqrt(xx*xx+yy*yy);
	X=X*X*X;

	f[0] = - K*xx/X;
	f[1] = - K*yy/X;
	f[2] = vx;
	f[3] = vy;

	return 0;  
}


WORKSPACE *workspace_alloc(int dim){
	WORKSPACE *w;

	w=(WORKSPACE *)malloc(sizeof(WORKSPACE));
	
	w->k1=(double *)malloc(dim*sizeof(double));
	w->k2=(double *)malloc(dim*sizeof(double));
	w->k3=(double *)malloc(dim*sizeof(double));
	w->k4=(double *)malloc(dim*sizeof(double));

	w->ytmp=(double *)malloc(dim*sizeof(double));

	return w;
}

void workspace_free(WORKSPACE *w){
	if(w!=NULL){
		free(w->k1);
		free(w->k2);
		free(w->k3);
		free(w->k4);
		free(w->ytmp);
		free(w);
	}
	
}

void rk4_step(double t, double *y, double h, FUNC(f), WORKSPACE *w, int n,void *p){

	int i;


	f(t,y,w->k1,n,p);
	for(i=0;i<n;i++) w->ytmp[i]=y[i]+0.5*h*w->k1[i];

	f(t+0.5*h, w->ytmp, w->k2,n,p);
	for(i=0;i<n;i++) w->ytmp[i]=y[i]+0.5*h*w->k2[i];

	f(t+0.5*h, w->ytmp, w->k3,n,p);
	for(i=0;i<n;i++) w->ytmp[i]=y[i]+h*w->k3[i];

	f(t+h, w->ytmp, w->k4,n, p);

	for(i=0;i<n;i++) y[i] = y[i] + 1.0/6.0*h*(w->k1[i]+2*w->k2[i]+2*w->k3[i]+w->k4[i]);


}

int main()
{
	WORKSPACE *w;
	int n=100;
	const int dim=4;
	double t,y[dim],y1[dim],y2[dim],h;
	double t1=0,t2;
	PARAMS p;
	int i,j,k;
	double E,L;
	double err,eps;

	p.M = 1.0;
	p.G = 1.0;

	

	//pocatecni podminky
	y[2]=0.0; //x=0;
	y[3]=10.0; // y=10;

	y[0]=sqrt(p.G*p.M/y[3]); //vx=0
	y[1]=0.0; //vy=0.1

	t2=2*PI*y[3]/y[0];
	
	t2=2*t2;

	h=(t2 - t1)/n;	

	t=0;
	
	printf("%lf ",t);
	for(j=0;j<dim;j++) printf("%lf ",y[j]);
	E=0.5*(y[0]*y[0]+y[1]*y[1]) - p.G*p.M/sqrt(y[2]*y[2]+y[3]*y[3]);
	L=sqrt(y[2]*y[2]+y[3]*y[3])*sqrt(y[0]*y[0]+y[1]*y[1]);
	printf("%.8lf %.8lf ",E,L);
	printf("\n");
		

	w=workspace_alloc(dim);
	h=1e-6;

	eps=1e-8;
	err=1;
	k=0;
	while(t<t2)
	{

		t=t+h;

		while(1)
		{		
			for(i=0;i<dim;i++) y1[i]=y[i];
			rk4_step(t,y1,2*h,&func,w,dim, &p);

			for(i=0;i<dim;i++) y2[i]=y[i];
			rk4_step(t,y2,h,&func,w,dim, &p);
			rk4_step(t+h,y2,h,&func,w,dim, &p);
		
			err=fabs(y1[2]-y2[2]);

			fprintf(stderr,"err %lf  eps %lf\n",err,eps);

			if(err>eps) h=h/2;
			else{ h=2*h;
			      break;
			}
		}		

		for(i=0;i<dim;i++) y[i]=y1[i];		

		printf("%lf ",t);
		for(j=0;j<dim;j++) printf("%lf ",y[j]);
		E=0.5*(y[0]*y[0]+y[1]*y[1]) - p.G*p.M/sqrt(y[2]*y[2]+y[3]*y[3]);
		L=sqrt(y[2]*y[2]+y[3]*y[3])*sqrt(y[0]*y[0]+y[1]*y[1]);	
		printf("%.8lf %.8lf ",E,L);
			printf("\n");

		k++;
	}

	fprintf(stderr,"num. of steps %d\n",k);
	workspace_free(w);

	return 0;
}







































