#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.1415927

#define FUNC(f) int (*f)(double , const double * , double *, int, void * )

typedef struct{
	double l;
	double q;
}PARAMS;

typedef struct{
	double *k1,*k2,*k3,*k4;
	double *ytmp;	
}WORKSPACE;


int func(double w, const double *y, double *f, int n, void *p)
{
	int i;
	PARAMS *par=(PARAMS *)p;
	double Uu,Um,u,m,t,phi;
	double l,l2,q;
	double u2,m2,dVu,dWm;
	
	Uu=y[0];
	Um=y[1];
	u=y[2];
	m=y[3];
	phi=y[4];
	t=y[5];

	l=par->l;l2=l*l;
	q=par->q;
	u2=u*u;
	m2=m*m;
	
	dVu=2*u2*u*(2 + (l2+q)*u2*(-3 + 7*u));
	dWm=-2*m*(l2+q);
	
	f[0] = 0.5*dVu;
	f[1] = 2*Um*Uu/u + 0.5*u2*u2*dWm;
	f[2] = Uu;
	f[3] = Um;
	f[4] = l*u2/(1-m2);
	f[5] = 1/(1-2*u);

	return 0;  
}


WORKSPACE *workspace_alloc(int dim){
	WORKSPACE *w;

	w=(WORKSPACE *)malloc(sizeof(WORKSPACE));
	
	w->k1=(double *)malloc(dim*sizeof(double));
	w->k2=(double *)malloc(dim*sizeof(double));
	w->k3=(double *)malloc(dim*sizeof(double));
	w->k4=(double *)malloc(dim*sizeof(double));

	w->ytmp=(double *)malloc(dim*sizeof(double));

	return w;
}

void workspace_free(WORKSPACE *w){
	if(w!=NULL){
		free(w->k1);
		free(w->k2);
		free(w->k3);
		free(w->k4);
		free(w->ytmp);
		free(w);
	}
	
}

void rk4_step(double t, double *y, double h, FUNC(f), WORKSPACE *w, int n,void *p){

	int i;


	f(t,y,w->k1,n,p);
	for(i=0;i<n;i++) w->ytmp[i]=y[i]+0.5*h*w->k1[i];

	f(t+0.5*h, w->ytmp, w->k2,n,p);
	for(i=0;i<n;i++) w->ytmp[i]=y[i]+0.5*h*w->k2[i];

	f(t+0.5*h, w->ytmp, w->k3,n,p);
	for(i=0;i<n;i++) w->ytmp[i]=y[i]+h*w->k3[i];

	f(t+h, w->ytmp, w->k4,n, p);

	for(i=0;i<n;i++) y[i] = y[i] + 1.0/6.0*h*(w->k1[i]+2*w->k2[i]+2*w->k3[i]+w->k4[i]);


}

int main()
{
	WORKSPACE *w;
	int n=10;
	const int dim=6;
	double t,y[dim],h;
	double t1=0,t2;
	PARAMS p;
	int i,j;
	
	p.l = 10.0;
	p.q = 0.0;

	

	//pocatecni podminky
	y[2]=1e-3;//u0
	y[3]=0.0;//m0
	y[4]=0.0;//phi0
	y[5]=0.0;//t0
	y[0]=y[2]*y[2]*sqrt(fabs(1-(1-2*y[2])*(p.l*p.l+p.q)*y[2]*y[2]));
	y[1]=y[2]*y[2]*sqrt(fabs(p.q-(p.l*p.l+p.q)*y[3]*y[3]));
	
	t2=20.0;
		


	h=(t2 - t1)/n;	

	t=0;
	
	printf("%lf ",t);
	for(j=0;j<dim;j++) printf("%lf ",y[j]);
	printf("\n");
		

	w=workspace_alloc(dim);
	
	for(i=1;i<=n;i++)
	{
		
		
		rk4_step(t,y,h,&func,w,dim, &p);
		
		t=i*h;
		
		printf("%lf ",t);
		for(j=0;j<dim;j++) printf("%lf ",y[j]);
		printf("\n");

	}

	workspace_free(w);

	return 0;
}







































